      SUBROUTINE LSQFIT(XPOINT,YPOINT,SIGDEV,NPOINT,
     +                  ACOEFF,MCOEFF,LISTAC,MC2FIT,
     +                  COVARS,NCVSIZ,CHISQR)

C     *************************************************************************
C
C     LSQFIT
C     ======
C
C     AUTHOR
C     ------
C     R.S.CANT  --  CAMBRIDGE UNIVERSITY ENGINEERING DEPARTMENT
C
C     CHANGE RECORD
C     -------------
C     13-OCT-2012:  CREATED
C      
C     DESCRIPTION
C     -----------
C     DOES A GENERAL LINEAR LEAST-SQUARES FIT
C     TO THE DATASET SPECIFIED BY THE NPOINT VALUES (XPOINT,YPOINT)
C     USING A SUBSET MC2FIT OF THE MCOEFF BASIS FUNCTIONS
C     SPECIFIED BY THE SUBROUTINE BFUNCT(XPOINT,ABFUNC,MCOEFF).
C     THE INDICES OF THE REQUIRED SUBSET ARE SPECIFED AS THE FIRST MC2FIT
C     ELEMENTS OF THE ARRAY LISTAC.
C     RETURNS THE MCOEFF COEFFICIENTS ACOEFF
C             THE GOODNESS-OF-FIT PARAMETER CHISQR
C             THE COVARIANCE MATRIX COVARS OF MAX SIZE (NCVSIZ,NCVSIZ)
C
C     BASED ON NUMERICAL RECIPES SUBROUTINE LFIT
C
C     *************************************************************************


C     PARAMETERS
C     ==========
      DOUBLE PRECISION ZERO,ONE
      PARAMETER(ZERO = 0.0D0, ONE = 1.0D0)

      INTEGER MCOFMX
      PARAMETER(MCOFMX = 10)


C     ARGUMENTS
C     =========
      DOUBLE PRECISION XPOINT(NPOINT),YPOINT(NPOINT),SIGDEV(NPOINT)
      DOUBLE PRECISION ACOEFF(MCOEFF)
      DOUBLE PRECISION COVARS(NCVSIZ,NCVSIZ)
      DOUBLE PRECISION CHISQR
      INTEGER LISTAC(MCOEFF)
      INTEGER NPOINT,MCOEFF,MC2FIT,NCVSIZ


C     LOCAL DATA
C     ==========
      DOUBLE PRECISION RHBETA(MCOFMX),ABFUNC(MCOFMX)
      DOUBLE PRECISION OVSIG2,YLOCAL,FOSIG2,SUMDEV
      INTEGER IC,JC,JJ,KC,KK,IHIT


C     BEGIN
C     =====

C     -------------------------------------------------------------------------

C     CHECK VALIDITY OF THE LIST OF COEFFICIENTS TO FIT
      KK = MC2FIT + 1
      DO JC = 1, MCOEFF
        IHIT = 0
        DO KC = 1, MC2FIT
          IF(LISTAC(KC).EQ.JC)IHIT = IHIT + 1
        ENDDO
        IF(IHIT.EQ.0)THEN
          LISTAC(KK) = JC
          KK = KK + 1
        ELSE IF(IHIT.GT.1)THEN
          WRITE(6,*) 'LSQFIT: improper list of coefficients supplied'
          STOP
        ENDIF
      ENDDO
      IF(KK.NE.(MCOEFF+1))THEN
        WRITE(6,*) 'LSQFIT: improper list of coefficients supplied'
        STOP
      ENDIF

C     -------------------------------------------------------------------------

C     INITIALISE THE MATRIX AND RHS VECTOR
      DO JC = 1, MC2FIT
        DO KC = 1, MC2FIT
          COVARS(JC,KC) = ZERO
        ENDDO
        RHBETA(JC) = ZERO
      ENDDO

C     -------------------------------------------------------------------------

C     ACCUMULATE COEFFICIENTS
      DO IC = 1, NPOINT

C       EVALUATE FITTING FUNCTIONS
        CALL BFUNCT(XPOINT(IC),ABFUNC,MCOEFF)
        YLOCAL = YPOINT(IC)

C       SUBTRACT THE KNOWN PART OF THE FITTING FUNCTION
        IF(MC2FIT.LT.MCOEFF)THEN
          DO JC = MC2FIT+1, MCOEFF
            JJ = LISTAC(JC)
            YLOCAL = YLOCAL - ACOEFF(JJ)*ABFUNC(JJ)
          ENDDO
        ENDIF

C       FILL IN THE MATRIX ELEMENTS
        OVSIG2 = SIGDEV(IC)
        OVSIG2 = OVSIG2*OVSIG2
        OVSIG2 = ONE/OVSIG2
        DO JC = 1, MC2FIT
          FOSIG2 = ABFUNC(LISTAC(JC))*OVSIG2
          DO KC = 1, JC
            COVARS(JC,KC) = COVARS(JC,KC) + FOSIG2*ABFUNC(LISTAC(KC))
          ENDDO
          RHBETA(JC) = RHBETA(JC) + FOSIG2*YLOCAL
        ENDDO

      ENDDO

C     SYMMETRIC MATRIX
      IF(MC2FIT.GT.1)THEN
        DO JC = 2, MC2FIT
          DO KC = 1, JC-1
            COVARS(KC,JC) = COVARS(JC,KC)
          ENDDO
        ENDDO
      ENDIF 

C     -------------------------------------------------------------------------

C     SOLVE THE MATRIX EQUATION
      CALL GAUSSJ(COVARS,MC2FIT,NCVSIZ,RHBETA,1,1)

C     -------------------------------------------------------------------------

C     PARTITION THE SOLUTION
      DO JC = 1, MC2FIT
        ACOEFF(LISTAC(JC)) = RHBETA(JC)
      ENDDO

C     -------------------------------------------------------------------------

C     GOODNESS OF FIT
      CHISQR = ZERO
      DO IC = 1, NPOINT
        CALL BFUNCT(XPOINT(IC),ABFUNC,MCOEFF)
        SUMDEV = ZERO
        DO JC = 1, MCOEFF
          SUMDEV = SUMDEV + ACOEFF(JC)*ABFUNC(JC)
        ENDDO
        CHISQR = CHISQR + ((YPOINT(IC)-SUMDEV)/SIGDEV(IC))**2
      ENDDO

C     -------------------------------------------------------------------------

C     SORT THE COVARANCE MATRIX
      CALL COVSRT(COVARS,NCVSIZ,MCOEFF,LISTAC,MC2FIT)

C     -------------------------------------------------------------------------


      RETURN
      END
